@@ -1,5 +1,4 @@
 from ...abstract_webtools import *
-
 from ..userAgentManager import *
 from ..cipherManager import *
 from ..sslManager import *
@@ -7,10 +6,14 @@ from ..tlsAdapter import *
 from ..networkManager import *
 from ..seleniumManager import *
 from ..urlManager import *
+logging.basicConfig(level=logging.INFO)
+
 class requestManager:
     """
-    SafeRequest is a class for making HTTP requests with error handling and retries.
-
+    requestManager is a class for making HTTP requests with error handling and retries.
+    It supports initializing with a provided source_code without requiring a URL.
+    If source_code is provided, it uses that as the response content and skips fetching.
+    Enhanced to parse source_code for URLs, PHP blocks, and React/JS data even if not HTML.
     Args:
         url (str or None): The URL to make requests to (default is None).
         url_mgr (urlManager or None): An instance of urlManager (default is None).
@@ -56,8 +59,7 @@ class requestManager:
         - The SafeRequest class is designed for making HTTP requests with error handling and retries.
         - It provides methods for authentication, response handling, and error management.
     """
-    def __init__(self,
-                 url=None,
+    def __init__(self,url=None,
                  source_code=None,
                  url_mgr=None,
                  network_manager=None,
@@ -78,256 +80,298 @@ class requestManager:
                  login_user_agent=None,
                  auth=None,
                  login_url=None,
-                 email = None,
+                 email=None,
                  password=None,
                  checkbox=None,
                  dropdown=None,
                  certification=None,
                  stream=False,
-                 timeout = None,
+                 timeout=None,
                  last_request_time=None,
                  max_retries=None,
-                 request_wait_limit=
-                 None):
-        self.url_mgr = get_url_mgr(url=url,url_mgr=url_mgr)
-        self.url=get_url(url=url,url_mgr=self.url_mgr)      
+                 request_wait_limit=None):
+        self.url_mgr = get_url_mgr(url=url, url_mgr=url_mgr)
+        self.url = get_url(url=url, url_mgr=self.url_mgr)
         self._url_mgr = self.url_mgr
-        self._url=self.url       
+        self._url = self.url
         self.user_agent = user_agent
         self.user_agent_manager = user_agent_manager or UserAgentManager(user_agent=self.user_agent)
-        self.headers= headers or self.user_agent_manager.header or {'Accept': '*/*'}
-        self.user_agent= self.user_agent_manager.user_agent
-        self.ciphers=ciphers or CipherManager().ciphers_string
-        self.certification=certification
-        self.ssl_options=ssl_options
+        self.headers = headers or self.user_agent_manager.header or {'Accept': '*/*'}
+        self.user_agent = self.user_agent_manager.user_agent
+        self.ciphers = ciphers or CipherManager().ciphers_string
+        self.certification = certification
+        self.ssl_options = ssl_options
         self.ssl_manager = ssl_manager or SSLManager(ciphers=self.ciphers, ssl_options=self.ssl_options, certification=self.certification)
-        self.tls_adapter=tls_adapter or  TLSAdapter(ssl_manager=self.ssl_manager,certification=self.certification,ssl_options=self.ssl_manager.ssl_options)
-        self.network_manager= network_manager or NetworkManager(user_agent_manager=self.user_agent_manager,ssl_manager=self.ssl_manager, tls_adapter=self.tls_adapter,user_agent=user_agent,proxies=proxies,cookies=cookies,ciphers=ciphers, certification=certification, ssl_options=ssl_options)
-        self.stream=stream
-        self.tls_adapter=self.network_manager.tls_adapter
-        self.ciphers=self.network_manager.ciphers
-        self.certification=self.network_manager.certification
-        self.ssl_options=self.network_manager.ssl_options
-        self.proxies=self.network_manager.proxies
-        self.timeout=timeout
-        self.cookies=self.network_manager.cookies
-        self.session = session or requests.session()
+        self.tls_adapter = tls_adapter or TLSAdapter(ssl_manager=self.ssl_manager, certification=self.certification, ssl_options=self.ssl_manager.ssl_options)
+        self.network_manager = network_manager or NetworkManager(user_agent_manager=self.user_agent_manager, ssl_manager=self.ssl_manager, tls_adapter=self.tls_adapter, user_agent=user_agent, proxies=proxies, cookies=cookies, ciphers=ciphers, certification=certification, ssl_options=ssl_options)
+        self.stream = stream
+        self.tls_adapter = self.network_manager.tls_adapter
+        self.ciphers = self.network_manager.ciphers
+        self.certification = self.network_manager.certification
+        self.ssl_options = self.network_manager.ssl_options
+        self.proxies = self.network_manager.proxies
+        self.timeout = timeout
+        self.cookies = self.network_manager.cookies
+        self.session = session or requests.Session()
         self.auth = auth
-        self.spec_login=spec_login
-        self.password=password
+        self.spec_login = spec_login
+        self.password = password
         self.email = email
-        self.checkbox=checkbox
-        self.dropdown=dropdown
-        self.login_url=login_url
-        self.login_user_agent=login_user_agent
-        self.login_referer=login_referer
-        self.protocol=protocol or 'https://'
-        
-        self.stream=stream if isinstance(stream,bool) else False
+        self.checkbox = checkbox
+        self.dropdown = dropdown
+        self.login_url = login_url
+        self.login_user_agent = login_user_agent
+        self.login_referer = login_referer
+        self.protocol = protocol or 'https://'
+        self.stream = stream if isinstance(stream, bool) else False
         self.initialize_session()
-        self.last_request_time=last_request_time
+        self.last_request_time = last_request_time
         self.max_retries = max_retries or 3
         self.request_wait_limit = request_wait_limit or 1.5
-        self._response=None
-        self.status_code=None
-        self.source_code = get_selenium_source(self.url)
-        self.source_code_bytes=None
+        self._response = None
+        self.status_code = None
+        self.source_code = None
+        self.source_code_bytes = None
         self.source_code_json = {}
-        self.react_source_code=[]
+        self.react_source_code = []
+        self.extracted_urls = []
+        self.php_blocks = []
         self._response_data = None
-        self.process_response_data()
-    def update_url_mgr(self,url_mgr):
-        self.url_mgr=url_mgr
+        if source_code is not None:
+            self._response = source_code
+            self.process_response_data()
+        else:
+            self.re_initialize()
+
+    def update_url_mgr(self, url_mgr):
+        self.url_mgr = url_mgr
         self.re_initialize()
-    def update_url(self,url):
+
+    def update_url(self, url):
         self.url_mgr.update_url(url=url)
         self.re_initialize()
+
     def re_initialize(self):
-        self._response=None
-        self.make_request()
+        self._response = None
+        if self.url_mgr.url is not None:
+            self.make_request()
         self.source_code = None
-        self.source_code_bytes=None
+        self.source_code_bytes = None
         self.source_code_json = {}
-        self.react_source_code=[]
+        self.react_source_code = []
+        self.extracted_urls = []
+        self.php_blocks = []
         self._response_data = None
         self.process_response_data()
+
     @property
     def response(self):
         """Lazy-loading of response."""
-        if self._response is None:
+        if self._response is None and self.url_mgr.url is not None:
             self._response = self.fetch_response()
-            
-            
         return self._response
-    def authenticate(self,session, login_url=None, email=None, password=None,checkbox=None,dropdown=None):
-        login_urls = login_url or [self.url_mgr.url,self.url_mgr.domain,self.url_mgr.url_join(url=self.url_mgr.domain,path='login'),self.url_mgr.url_join(url=self.url_mgr.domain,path='auth')]
+
+    def authenticate(self, session, login_url=None, email=None, password=None, checkbox=None, dropdown=None):
+        login_urls = login_url or [self.url_mgr.url, self.url_mgr.domain, self.url_mgr.url_join(url=self.url_mgr.domain, path='login'), self.url_mgr.url_join(url=self.url_mgr.domain, path='auth')]
         s = session
-        if not isinstance(login_urls,list):
-            login_urls=[login_urls]
+        if not isinstance(login_urls, list):
+            login_urls = [login_urls]
         for login_url in login_urls:
             login_url_mgr = urlManager(login_url)
             login_url = login_url_mgr.url
-            
             r = s.get(login_url)
             soup = BeautifulSoup(r.content, "html.parser")
             # Find the token or any CSRF protection token
             token = soup.find('input', {'name': 'token'}).get('value') if soup.find('input', {'name': 'token'}) else None
-            if token != None:
+            if token is not None:
                 break
         login_data = {}
-        if email != None:
-            login_data['email']=email
-        if password != None:
+        if email is not None:
+            login_data['email'] = email
+        if password is not None:
             login_data['password'] = password
-        if checkbox != None:
+        if checkbox is not None:
             login_data['checkbox'] = checkbox
-        if dropdown != None:
-            login_data['dropdown']=dropdown
-        if token != None:
+        if dropdown is not None:
+            login_data['dropdown'] = dropdown
+        if token is not None:
             login_data['token'] = token
         s.post(login_url, data=login_data)
         return s
 
-    def fetch_response(self) -> Union[requests.Response, None]:
+    def fetch_response(self) -> requests.Response | None | str | bytes:
         """Actually fetches the response from the server."""
-        # You can further adapt this method to use retries or other logic you had
-        # in your original code, but the main goal here is to fetch and return the response
         return self.try_request()
+
     def spec_auth(self, session=None, email=None, password=None, login_url=None, login_referer=None, login_user_agent=None):
-        s = session or requests.session()
-        
-        domain = self.url_mgr.url_join(self.url_mgr.get_correct_url(self.url_mgr.domain),'login') if login_url is None else login_url
+        s = session or requests.Session()
+        domain = self.url_mgr.url_join(self.url_mgr.get_correct_url(self.url_mgr.domain), 'login') if login_url is None else login_url
         login_url = self.url_mgr.get_correct_url(url=domain)
-        
         login_referer = login_referer or self.url_mgr.url_join(url=login_url, path='?role=fast&to=&s=1&m=1&email=YOUR_EMAIL')
         login_user_agent = login_user_agent or 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.12; rv:50.0) Gecko/20100101 Firefox/50.0'
-        
         headers = {"Referer": login_referer, 'User-Agent': login_user_agent}
         payload = {'email': email, 'pass': password}
-        
         page = s.get(login_url)
         soup = BeautifulSoup(page.content, 'lxml')
         action_url = soup.find('form')['action']
         s.post(action_url, data=payload, headers=headers)
         return s
+
     def initialize_session(self):
-        s = self.session  
+        s = self.session
         if self.auth:
-            s= self.auth
+            s = self.auth
         elif self.spec_login:
-            s=self.spec_auth(session=s,email=self.email, password=self.password, login_url=self.login_url, login_referer=self.login_referer, login_user_agent=self.login_user_agent)
+            s = self.spec_auth(session=s, email=self.email, password=self.password, login_url=self.login_url, login_referer=self.login_referer, login_user_agent=self.login_user_agent)
         elif any([self.password, self.email, self.login_url, self.checkbox, self.dropdown]):
-            s=self.authenticate(session=s, login_url=self.login_url, email=self.email, password=self.password, checkbox=self.checkbox, dropdown=self.dropdown)
+            s = self.authenticate(session=s, login_url=self.login_url, email=self.email, password=self.password, checkbox=self.checkbox, dropdown=self.dropdown)
         s.proxies = self.proxies
         s.cookies["cf_clearance"] = self.network_manager.cookies
         s.headers.update(self.headers)
         s.mount(self.protocol, self.network_manager.tls_adapter)
         return s
+
     def process_response_data(self):
         """Processes the fetched response data."""
         if not self.response:
             return  # No data to process
-        if  isinstance(self.response,str):
-            self.source_code = self.response
+        if isinstance(self.response, (str, bytes)):
+            if isinstance(self.response, str):
+                self.source_code = self.response
+                self.source_code_bytes = self.response.encode('utf-8')  # Assume UTF-8
+            else:
+                self.source_code_bytes = self.response
+                try:
+                    self.source_code = self.response.decode('utf-8')
+                except UnicodeDecodeError:
+                    self.source_code = self.response.decode('latin-1')  # Fallback
+            # Check if it's JSON
+            try:
+                data = json.loads(self.source_code)
+                self.source_code_json = data.get("response", data)
+            except json.JSONDecodeError:
+                pass
         else:
             self.source_code = self.response.text
             self.source_code_bytes = self.response.content
-            if self.response.headers.get('content-type') == 'application/json':
-                data = convert_to_json(self.source_code)
-                if data:
+            if self.response.headers.get('content-type', '').startswith('application/json'):
+                try:
+                    data = json.loads(self.source_code)
                     self.source_code_json = data.get("response", data)
-            
-            self.get_react_source_code()
+                except json.JSONDecodeError:
+                    pass
+        self.extract_urls()
+        self.extract_php_blocks()
+        self.get_react_source_code()
+
+    def extract_urls(self):
+        """Extract URLs from source_code using regex."""
+        if not self.source_code:
+            return
+        url_pattern = r'https?://[^\s<>"\']+'
+        self.extracted_urls = re.findall(url_pattern, self.source_code)
+
+    def extract_php_blocks(self):
+        """Extract PHP blocks from source_code if present."""
+        if not self.source_code:
+            return
+        php_pattern = r'<\?php(.*?)?\?>'
+        self.php_blocks = re.findall(php_pattern, self.source_code, re.DOTALL)
+
     def get_react_source_code(self) -> list:
         """
-        Fetches the source code of the specified URL and extracts JavaScript and JSX source code (React components).
-
-        Args:
-            url (str): The URL to fetch the source code from.
-
-        Returns:
-            list: A list of strings containing JavaScript and JSX source code found in <script> tags.
+        Extracts JavaScript and JSX source code from <script> tags if HTML-like.
+        If not HTML and looks like JS/React code, appends the whole source_code.
         """
-        if self.url_mgr.url is None:
+        if not self.source_code:
             return []
-        soup = BeautifulSoup(self.source_code_bytes,"html.parser")
-        script_tags = soup.find_all('script', type=lambda t: t and ('javascript' in t or 'jsx' in t))
+        # Check if likely JS code (e.g., contains 'import', 'function', 'React')
+        is_js_like = any(keyword in self.source_code.lower() for keyword in ['import ', 'function ', 'react', 'export ', 'const ', 'let ', 'var '])
+        # Check if HTML-like
+        is_html_like = self.source_code.strip().startswith('<') or '<html' in self.source_code.lower() or '<!doctype' in self.source_code.lower()
+        if not is_html_like and is_js_like:
+            self.react_source_code.append(self.source_code)
+            return self.react_source_code
+        content = self.source_code_bytes or self.source_code.encode('utf-8')
+        soup = BeautifulSoup(content, "html.parser")
+        script_tags = soup.find_all('script', type=lambda t: t and ('javascript' in t.lower() or 'jsx' in t.lower()))
         for script_tag in script_tags:
-            self.react_source_code.append(script_tag.string)
+            if script_tag.string:
+                self.react_source_code.append(script_tag.string)
+        # If no scripts found but JS-like, append whole
+        if not script_tags and is_js_like:
+            self.react_source_code.append(self.source_code)
+        return self.react_source_code
 
-
-    def get_status(url:str=None) -> int:
+    def get_status(self, url: str = None) -> int | None:
         """
         Gets the HTTP status code of the given URL.
-
-        Args:
-            url (str): The URL to check the status of.
-
-        Returns:
-            int: The HTTP status code of the URL, or None if the request fails.
         """
-        # Get the status code of the URL
-        return try_request(url=url).status_code
+        url = url or self.url_mgr.url
+        if url is None:
+            return None
+        try:
+            response = requests.head(url, timeout=5)
+            return response.status_code
+        except requests.RequestException:
+            return None
+
     def wait_between_requests(self):
         """
         Wait between requests based on the request_wait_limit.
         """
         if self.last_request_time:
-            sleep_time = self.request_wait_limit - (get_time_stamp() - self.last_request_time)
+            sleep_time = self.request_wait_limit - (time.time() - self.last_request_time)
             if sleep_time > 0:
                 logging.info(f"Sleeping for {sleep_time:.2f} seconds.")
-                get_sleep(sleep_time)
+                time.sleep(sleep_time)
 
     def make_request(self):
         """
         Make a request and handle potential errors.
         """
-        # Update the instance attributes if they are passed
-        
+        if self.url_mgr.url is None:
+            return None
         self.wait_between_requests()
         for _ in range(self.max_retries):
             try:
-                self.try_request()  # 10 seconds timeout
-                if self.response:
-                    self.status_code = self.response.status_code
-                    if self.response.status_code == 200:
-                        self.last_request_time = get_time_stamp()
-                        return self.response
-                    elif self.response.status_code == 429:
-                        logging.warning(f"Rate limited by {self.url_mgr.url}. Retrying...")
-                        get_sleep(5)  # adjust this based on the server's rate limit reset time
+                self._response = self.try_request()
+                if self._response:
+                    if not isinstance(self._response, (str, bytes)):
+                        self.status_code = self._response.status_code
+                        if self._response.status_code == 200:
+                            self.last_request_time = time.time()
+                            return self._response
+                        elif self._response.status_code == 429:
+                            logging.warning(f"Rate limited by {self.url_mgr.url}. Retrying...")
+                            time.sleep(5)
             except requests.Timeout as e:
-                logging.error(f"Request to {cleaned_url} timed out: {e}")
+                logging.error(f"Request to {self.url_mgr.url} timed out: {e}")
             except requests.ConnectionError:
                 logging.error(f"Connection error for URL {self.url_mgr.url}.")
-            except requests.Timeout:
-                logging.error(f"Request timeout for URL {self.url_mgr.url}.")
             except requests.RequestException as e:
                 logging.error(f"Request exception for URL {self.url_mgr.url}: {e}")
         try:
             response = get_selenium_source(self.url_mgr.url)
             if response:
-                self.response = response
-                return self.response
-        except:
-            logging.error(f"Failed to retrieve content from {self.url_mgr.url} after {self.max_retries} retries.")
+                self._response = response
+                self.status_code = 200  # Assume success
+                return self._response
+        except Exception as e:
+            logging.error(f"Failed to retrieve content from {self.url_mgr.url} after {self.max_retries} retries: {e}")
             return None
-    def try_request(self) -> Union[requests.Response, None]:
+
+    def try_request(self) -> requests.Response | str | bytes | None:
         """
         Tries to make an HTTP request to the given URL using the provided session.
-
-        Args:
-            timeout (int): Timeout for the request.
-
-        Returns:
-            requests.Response or None: The response object if the request is successful, or None if the request fails.
         """
+        if self.url_mgr.url is None:
+            return None
         try:
-            return get_selenium_source(self.url_mgr.url)#self.session.get(url=self.url_mgr.url, timeout=self.timeout,stream=self.stream)
-        except requests.exceptions.RequestException as e:
-            print(e)
+            return get_selenium_source(self.url_mgr.url)  # or self.session.get(self.url_mgr.url, timeout=self.timeout, stream=self.stream)
+        except requests.RequestException as e:
+            logging.error(f"Request failed: {e}")
             return None
-
 
     @property
     def url(self):
@@ -345,12 +389,11 @@ class SafeRequestSingleton:
         elif SafeRequestSingleton._instance.url != url or SafeRequestSingleton._instance.headers != headers or SafeRequestSingleton._instance.max_retries != max_retries or SafeRequestSingleton._instance.request_wait_limit != request_wait_limit:
             SafeRequestSingleton._instance = SafeRequest(url,url_mgr=urlManagerSingleton,headers=headers,max_retries=max_retries,last_request_time=last_request_time,request_wait_limit=request_wait_limit)
         return SafeRequestSingleton._instance
+def get_source(url=None,url_mgr=None,source_code=None,req_mgr=None):
+    req_mgr = get_req_mgr(req_mgr=req_mgr,url=url,url_mgr=url_mgr,source_code=source_code)
+    return req_mgr.source_code
 def get_req_mgr(url=None,url_mgr=None,source_code=None,req_mgr=None):
     url = get_url(url=url,url_mgr=url_mgr)
     url_mgr = get_url_mgr(url=url,url_mgr=url_mgr )
     req_mgr = req_mgr  or requestManager(url_mgr=url_mgr,url=url,source_code=source_code)
     return req_mgr
-def get_source(url=None,url_mgr=None,source_code=None,req_mgr=None):
-    # Placeholder for actual implementation.
-    req_mgr = get_req_mgr(req_mgr=req_mgr,url=url,url_mgr=url_mgr,source_code=source_code)
-    return req_mgr.source_code
