@@ -39,7 +39,7 @@ class soupManager:
         - The SoupManager class is designed for parsing HTML source code using BeautifulSoup.
         - It provides various methods to extract data and discover elements within the source code.
     """
-    def __init__(self,url=None,source_code=None,url_mgr=None,req_mgr=None, parse_type="html.parser"):
+    def __init__(self,url=None,source_code=None,url_mgr=None,req_mgr=None,soup=None, parse_type="html.parser"):
         self.soup=[]
         url = get_url(url=url,url_mgr=url_mgr)
         self.url_mgr = get_url_mgr(url=url,url_mgr=url_mgr)
@@ -49,8 +49,8 @@ class soupManager:
         source_code = source_code or self.req_mgr.source_code or self.req_mgr.source_code_bytes
         if source_code:
             source_code = str(source_code)
-        self.source_code = source_code
-        self.soup= BeautifulSoup(self.source_code, self.parse_type)
+        self.source_code = source_code or ''
+        self.soup= soup or BeautifulSoup(self.source_code, self.parse_type)
         self.all_tags_and_attribute_names = self.get_all_tags_and_attribute_names()
         self.all_tags = self.all_tags_and_attribute_names.get('tags')
         self.all_attribute_names = self.all_tags_and_attribute_names.get('attributes')
@@ -340,16 +340,8 @@ class SoupManagerSingleton():
         elif parse_type != SoupManagerSingleton._instance.parse_type  or source_code != SoupManagerSingleton._instance.source_code:
             SoupManagerSingleton._instance = SoupManager(url_mgr,requestManager,parse_type=parse_type,source_code=source_code)
         return SoupManagerSingleton._instance
-def get_soup_mgr(url=None,url_mgr=None,source_code=None,req_mgr=None,soup_mgr=None,parse_type="html.parser"):
-    url_mgr = get_url_mgr(url=url,url_mgr=url_mgr)
-    url = get_url(url=url,url_mgr=url_mgr)
-    req_mgr = get_req_mgr(url_mgr=url_mgr,url=url,source_code=source_code)
-    soup_mgr = soup_mgr or soupManager(url_mgr=url_mgr,req_mgr=req_mgr,url=url,source_code=source_code)
-    return soup_mgr
-def get_all_attribute_values(url=None,url_mgr=None,source_code=None,req_mgr=None,soup_mgr=None,tags_list = None,parse_type="html.parser"):
-    soup_mgr = get_soup_mgr(url=url,url_mgr=url_mgr,source_code=source_code,req_mgr=req_mgr,soup_mgr=soup_mgr)
-    return soup_mgr.get_all_attribute_values(tags_list=tags_list)
-def get_soup(url=None,url_mgr=None,req_mgr=None,source_code=None,soup_mgr=None,parse_type="html.parser"):
+def get_soup(url=None,url_mgr=None,req_mgr=None,source_code=None,soup_mgr=None,soup=None,parse_type=None):
+    parse_type = parse_type or "html.parser"
     if source_code or soup_mgr:
         if soup_mgr:
             return soup_mgr.soup
@@ -358,5 +350,16 @@ def get_soup(url=None,url_mgr=None,req_mgr=None,source_code=None,soup_mgr=None,p
     url = get_url(url=url,url_mgr=url_mgr)
     req_mgr = req_mgr or get_req_mgr(url_mgr=url_mgr,url=url,source_code=source_code)
     source_code = req_mgr.source_code
-    soup_mgr = get_soup_mgr(url=url,url_mgr=url_mgr,source_code=source_code,req_mgr=req_mgr,soup_mgr=soup_mgr)
+    soup_mgr = get_soup_mgr(url=url,url_mgr=url_mgr,source_code=source_code,req_mgr=req_mgr,soup_mgr=soup_mgr,soup=soup)
     return soup_mgr.soup
+def get_soup_mgr(url=None,url_mgr=None,source_code=None,req_mgr=None,soup_mgr=None,soup=None,parse_type=None):
+    parse_type = parse_type or "html.parser"
+    url_mgr = get_url_mgr(url=url,url_mgr=url_mgr)
+    url = get_url(url=url,url_mgr=url_mgr)
+    req_mgr = get_req_mgr(url_mgr=url_mgr,url=url,source_code=source_code)
+    soup_mgr = soup_mgr or soupManager(url_mgr=url_mgr,req_mgr=req_mgr,url=url,source_code=source_code,soup=soup)
+    return soup_mgr
+def get_all_attribute_values(url=None,url_mgr=None,source_code=None,req_mgr=None,soup_mgr=None,soup=None,tags_list = None,parse_type=None):
+    parse_type = parse_type or "html.parser"
+    soup_mgr = get_soup_mgr(url=url,url_mgr=url_mgr,source_code=source_code,req_mgr=req_mgr,soup_mgr=soup_mgr,soup=soup)
+    return soup_mgr.get_all_attribute_values(tags_list=tags_list)
